#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "lzpt_io.h"

#if	defined(__POCC__)
extern	int	_unlink(const char *fname);
#define	unlink(s)	_unlink(s)
#endif

int main(int argc, char *argv[])
{
	FILE	*fh_in = NULL, *fh_out = NULL;
	char	*fname_in, *fname_out;
	unsigned char	*p_toc = NULL;
	unsigned char	*p_enc_block = NULL, *p_dec_block = NULL;
	size_t	sz_enc_block, sz_dec_block;
	int	toc_nentries;
	int	i;
	size_t	filesize;
	int	retval;


	if (argc != 3) {
		fprintf(stderr, "Usage: %s infile outfile\n", argv[0]);
		exit(1);
	}

	fname_in = argv[1];
	fname_out = argv[2];
	
	if (!(fh_in = fopen(fname_in, "rb"))) {
		fprintf(stderr, "Error opening input file '%s'!\n", fname_in);
		goto exit_err;
	}
	fseek(fh_in, 0L, SEEK_END);
	filesize = ftell(fh_in);
	fseek(fh_in, 0L, SEEK_SET);


	if (lzpt_read_toc(fh_in, &p_toc, &toc_nentries)) {
		fprintf(stderr, "Error reading TOC!\n");
		goto exit_err;
	}

	printf("Decompressing '%s' (%d blocks) to '%s' ...", fname_in, toc_nentries, fname_out); fflush(stdout);

	if (!(fh_out = fopen(fname_out, "wb"))) {
		fprintf(stderr, "Error creating output file '%s'!\n", fname_out);
		goto exit_err;
	}

	for (i = 0; i < toc_nentries; i++) {
		if (lzpt_read_block(fh_in, i, p_toc, toc_nentries, &p_enc_block, &sz_enc_block)) {
			fprintf(stderr, "Error reading LZTP block #%d\n", i);
			goto exit_err;
		}
		if (lzpt_decompress_block(p_enc_block, sz_enc_block, &p_dec_block, &sz_dec_block)) {
			fprintf(stderr, "Error decompressing LZTP block #%d\n", i);
			goto exit_err;
		}
		free((void *)p_enc_block); p_enc_block = NULL;

		if (fwrite(p_dec_block, 1, sz_dec_block, fh_out) != sz_dec_block) {
			fprintf(stderr, "Error writing LZTP block #%d\n", i);
			goto exit_err;
		}
		free((void *)p_dec_block); p_dec_block = NULL;
	}

	fclose(fh_in); fh_in = NULL;
	fclose(fh_out); fh_out = NULL;

	printf("Done\n");
	goto exit_ok;


exit_err:
	if (fh_in) fclose(fh_in);
	if (fh_out) fclose(fh_out);
	unlink(fname_out);
	retval = 1;
	goto exit_common;

exit_ok:
	retval = 0;
	// fallthru

exit_common:
	if (p_toc) lzpt_free_toc(p_toc, toc_nentries);
	if (p_enc_block) free(p_enc_block);
	if (p_dec_block) free(p_dec_block);

	return retval;
}
